package com.ims.eval.service.impl;

import com.ims.common.utils.Constant;
import com.ims.common.utils.FormulaUtils;
import com.ims.common.utils.StringUtils;
import com.ims.eval.config.CustomException;
import com.ims.eval.dao.CalculateIndicatorItemInfoMapper;
import com.ims.eval.entity.*;
import com.ims.eval.dao.OrganizationEvaluationInfoMapper;
import com.ims.eval.entity.dto.response.MyuserResDTO;
import com.ims.eval.entity.dto.response.OrganizationEvaluationInfoResDTO;
import com.ims.eval.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 考评指标明细

 服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-03-16
 */
@Service
@Log4j2
public class OrganizationEvaluationInfoServiceImpl extends ServiceImpl<OrganizationEvaluationInfoMapper, OrganizationEvaluationInfo> implements IOrganizationEvaluationInfoService {

	@Autowired
	private ICalculateIndicatorItemInfoService iCalculateIndicatorItemInfoService;


	@Autowired
	private IIndicatorDictionaryService indicatorDictionaryService;

	@Autowired
	private IUserService userService;

	@Autowired
	private IEvaluationScoreCountService iEvaluatioinScoreCountService;


	@Autowired
	private CalculateIndicatorItemInfoMapper calculateIndicatorItemInfoMapper;

	@Override
	public Map finishValueList(String organizationEvaluationId, String dept, HttpServletRequest request) {

		MyuserResDTO myuser = userService.getSysUser(request);
		if(null == myuser){
			throw new CustomException("暂无权限");
		}
		String organizationId = "";
		boolean updateMark = false;
		if (null != myuser.getUnitOrg() && null != myuser.getUnitOrg().getParentIds()) {
			String type = myuser.getUnitOrg().getType();
			if (Constant.THREE.contains(type) || Constant.FOUR.contains(type)) {
//				dept = myuser.getDeptName();
				organizationId = myuser.getUnitId();
			} else if (Constant.SUB_BRANCH.contains(type)) {
				dept = "";
			} else if (myuser.getLoginName().equals("admin")) {
				dept = "";
				updateMark= true;
			} else {
				throw new CustomException("暂无权限");
			}
		}
		List<OrganizationEvaluationInfoResDTO> list = baseMapper.selectListEvaluationInfoId(organizationEvaluationId,dept,"",organizationId);//默认获取指定的怎目标
		boolean finalUpdateMark = updateMark;
		list.stream().forEach(i->{
			i.setUpdateMark(true);
		});
		Map<String  ,List<OrganizationEvaluationInfoResDTO>> map  = list.stream().collect(Collectors.groupingBy(OrganizationEvaluationInfoResDTO::getIsQuantified));
		map.put("quantifiedList", map.remove("是"));
		map.put("nonQuantifiedList", map.remove("否"));
		return map;
	}

	@Transactional
	@Override
	public boolean saveBatchDto(List<OrganizationEvaluationInfo> organizationEvaluationInfoList) throws Exception {
		for (OrganizationEvaluationInfo entity : organizationEvaluationInfoList) {

			if (null != entity && null != entity.getId() && !"".equals(entity.getId().trim())) {
				OrganizationEvaluationInfo info = baseMapper.selectById(entity.getId());

				info.setQuantifiedValue(entity.getQuantifiedValue());//量化值
				if("JKF".equals(info.getOptionCode())){//加扣分
					if(StringUtils.isNumber(entity.getNonQuantifiedValue())){
						info.setQuantifiedValue(Double.valueOf(entity.getNonQuantifiedValue()));
					}
				}

				if("JCF".equals(info.getOptionCode())){//基础分
					if(StringUtils.isNumber(entity.getNonQuantifiedValue())){
						info.setQuantifiedValue(Double.valueOf(entity.getNonQuantifiedValue()));
					}
				}

				info.setNonQuantifiedValue(entity.getNonQuantifiedValue());//非量化值
				info.setUpdateTime(new Date());//更新时间
				info.setUpdateBy(entity.getUpdateBy());//更新者
				info.setRemark(entity.getRemark());//备注
				info.setState(entity.getState());//初始状态
				boolean b = super.saveOrUpdate(info);
				if(!b){
					throw new CustomException("更新失败");
				}
			}else {
				entity.setCreateTime(new Date());

				if (StringUtils.isEmpty(entity.getIndicatorId())) {
					throw new CustomException("保存失败");
				}
				List<IndicatorDictionary> dictionaryList =  indicatorDictionaryService.list("",entity.getIndicatorId(),"","");
				for(IndicatorDictionary d : dictionaryList){

					OrganizationEvaluationInfo addinfo = new OrganizationEvaluationInfo();
					addinfo.setOrganizationEvaluationId(entity.getOrganizationEvaluationId());
					addinfo.setOrganizationEvaluationRuleId(entity.getOrganizationEvaluationRuleId());
					addinfo.setIndicatorId(entity.getIndicatorId());
					addinfo.setIndicatorDictionaryId(d.getId());
					addinfo.setOptionCode(d.getOptionCode());
					addinfo.setChildCode(d.getChildCode());
					addinfo.setIsQuantified(entity.getIsQuantified());
					addinfo.setQuantifiedValue(entity.getQuantifiedValue());
					addinfo.setNonQuantifiedValue(entity.getNonQuantifiedValue());
					addinfo.setState("-1");//初始状态
					addinfo.setCreateTime(new Date());

					boolean b = super.saveOrUpdate(addinfo);
					if(!b){
						throw new CustomException("保存失败");
					}
				}

			}

		}

		OrganizationEvaluationInfo organizationEvaluationInfo = baseMapper.selectById(organizationEvaluationInfoList.get(0).getId());
		//查询本次考评所有考评指标明细
		List<OrganizationEvaluationInfo> organizationEvaluationInfos = baseMapper.selectByEvaluationInfoId(organizationEvaluationInfo.getOrganizationEvaluationId());

		//调用需要计算指标项的方法
		boolean calculate = calculateIdicatorItem(organizationEvaluationInfos);

		//调用计算存储最大值，最小值的方法
		if (calculate){
			iCalculateIndicatorItemInfoService.saveIndicatorInfo(organizationEvaluationInfos);
		}


		return true;
	}






	@Override
	public List<OrganizationEvaluationInfoResDTO> finishValueList(String organizationEvaluationId, String dept, String organizationShortName, String indicatorName, String binSection,String isQuantified, HttpServletRequest request) {
		MyuserResDTO myuser = userService.getSysUser(request);
		if(null == myuser){
			throw new CustomException("暂无权限");
		}
		String organizationId = "";
		boolean updateMark = false;
		if (null != myuser.getUnitOrg() && null != myuser.getUnitOrg().getParentIds()) {
			String type = myuser.getUnitOrg().getType();
			if (Constant.THREE.contains(type) || Constant.FOUR.contains(type)) {
//				dept = myuser.getDeptName();
				organizationId = myuser.getUnitId();
			} else if (Constant.SUB_BRANCH.contains(type)) {
				dept = "";
			} else if (myuser.getLoginName().equals("admin")) {
				dept = "";
				updateMark= true;
			} else {
				throw new CustomException("暂无权限");
			}
		}
		List<OrganizationEvaluationInfoResDTO> list = baseMapper.selectListEvaluationInfoId2(organizationEvaluationId,dept,organizationShortName,indicatorName,binSection,isQuantified,"",organizationId);//默认获取指定的怎目标
		list.stream().forEach(i->{
			i.setUpdateMark(true);
		});
		return list;
	}

	@Override
	public boolean calculationByEvaluationId(String id, String indicatorId,String organizationEvaluationId,String optionCode) {
		try {
			List<OrganizationEvaluationInfoResDTO> list = baseMapper.selectByIdAndEvaluationInfoId(id,indicatorId,organizationEvaluationId,"");
			//根据板块分组
			Map<String, List<OrganizationEvaluationInfoResDTO>> binSectionGropList = list.stream().collect(Collectors.groupingBy(OrganizationEvaluationInfoResDTO::getBinSection));

			for (Map.Entry<String, List<OrganizationEvaluationInfoResDTO>> binSectionGropEntry : binSectionGropList.entrySet()){
				if (binSectionGropEntry.getValue().size() > 0){
					//按照指标分组
					Map<String, List<OrganizationEvaluationInfoResDTO>> indicatorGropList = binSectionGropEntry.getValue().stream().collect(Collectors.groupingBy(OrganizationEvaluationInfoResDTO::getIndicatorId));


					//计算前置属性（差值、利润贡献、单位千瓦利润）的最大最小值
					for (Map.Entry<String, List<OrganizationEvaluationInfoResDTO>> indicatorGropListEntry : indicatorGropList.entrySet()){

						if (indicatorGropListEntry.getValue().size() > 0){
							//按照公司规则分组
							Map<String, List<OrganizationEvaluationInfoResDTO>> ruleIdGropList = indicatorGropListEntry.getValue().stream().collect(Collectors.groupingBy(OrganizationEvaluationInfoResDTO::getOrganizationEvaluationRuleId));

							//遍历公司指标
							for (Map.Entry<String, List<OrganizationEvaluationInfoResDTO>> ruleIdGropListEntry : ruleIdGropList.entrySet()){
								if (ruleIdGropListEntry.getValue().size() > 0) {

									List<OrganizationEvaluationInfoResDTO> indicator = ruleIdGropListEntry.getValue();
									Map<String, List<OrganizationEvaluationInfoResDTO>> childCodeGropList = indicator.stream().collect(Collectors.groupingBy(OrganizationEvaluationInfoResDTO::getChildCode));

									double totalScore = 0.00;
									for (Map.Entry<String, List<OrganizationEvaluationInfoResDTO>> childCodeGropListEntry : childCodeGropList.entrySet()){
										for (OrganizationEvaluationInfoResDTO dto : childCodeGropListEntry.getValue()) {

											if ((!dto.getOptionCode().equals("CZ") && !dto.getOptionCode().equals("LRGXL") && !dto.getOptionCode().equals("DWQWLR")) || StringUtils.isEmpty(dto.getFormula())) {
												continue;//当前属性没有配置公式跳过
											}

											for (OrganizationEvaluationInfoResDTO f : childCodeGropListEntry.getValue()) {
												dto.setFormula(dto.getFormula().replace("[" + f.getOptionCode() + "]", f.getQuantifiedValue() + ""));
											}
											double score = FormulaUtils.caculateFormula(dto.getFormula());
											if (StringUtils.inStringIgnoreCase("DF", dto.getOptionCode())) {
												totalScore = totalScore + score;
											}
											log.info(dto.getOrganizationShortName() + "|" + dto.getChildName()+"|"+dto.getOptionCode() + "----------" + dto.getFormula() + "=" + score);
											OrganizationEvaluationInfo info = baseMapper.selectById(dto.getId());
											//将计算结果保存
											info.setQuantifiedValue(score);
											dto.setQuantifiedValue(score);
											this.saveOrUpdate(info);
										}
								}
								}
							}
						}
						//计算差值等的最大值最小值
						iCalculateIndicatorItemInfoService.saveIndicatorInfoDTO(indicatorGropListEntry.getValue());
					}


//					//计算前置属性（差值、利润贡献、单位千瓦利润）的最大最小值
					for (Map.Entry<String, List<OrganizationEvaluationInfoResDTO>> indicatorGropListEntry : indicatorGropList.entrySet()){
						for (OrganizationEvaluationInfoResDTO d : indicatorGropListEntry.getValue()){
							System.out.println(d.getOrganizationShortName()+"|"+d.getIndicatorName()+"|"+d.getOptionName()+"|"+d.getQuantifiedValue());
						}

						if (indicatorGropListEntry.getValue().size() > 0){
							//按照公司规则分组
							Map<String, List<OrganizationEvaluationInfoResDTO>> ruleIdGropList = indicatorGropListEntry.getValue().stream().collect(Collectors.groupingBy(OrganizationEvaluationInfoResDTO::getOrganizationEvaluationRuleId));


							//遍历公司指标
							for (Map.Entry<String, List<OrganizationEvaluationInfoResDTO>> ruleIdGropListEntry : ruleIdGropList.entrySet()){
								if (ruleIdGropListEntry.getValue().size() > 0){

									List<OrganizationEvaluationInfoResDTO> indicator = ruleIdGropListEntry.getValue();
									Map<String, List<OrganizationEvaluationInfoResDTO>> childCodeGropList = indicator.stream().collect(Collectors.groupingBy(OrganizationEvaluationInfoResDTO::getChildCode));
									double totalScore = 0.00;
									EvaluationScoreCount scoreCount = new EvaluationScoreCount();

									for (Map.Entry<String, List<OrganizationEvaluationInfoResDTO>> childCodeGropListEntry : childCodeGropList.entrySet()){

										List<CalculateIndicatorItemInfo> calculateIndicatorItemInfo = calculateIndicatorItemInfoMapper.selectList("","","","","",childCodeGropListEntry.getValue().get(0).getOrganizationEvaluationId(),childCodeGropListEntry.getValue().get(0).getChildCode());
										Map<String ,Double>  map = new HashMap<>();
										for (CalculateIndicatorItemInfo itemInfo :calculateIndicatorItemInfo){
											map.put(itemInfo.getOptionCode(),itemInfo.getQuantifiedValue());
										}
										for (OrganizationEvaluationInfoResDTO dto: childCodeGropListEntry.getValue()){
											if ((dto.getOptionCode().endsWith("MAX") ||dto.getOptionCode().endsWith("MIN")
												||dto.getOptionCode().equals("CZ") || dto.getOptionCode().equals("LRGXL")
												|| dto.getOptionCode().equals("DWQWLR")) || StringUtils.isEmpty(dto.getFormula())){
												continue;//当前属性没有配置公式跳过或者不需要参与此次计算的属性
											}
											scoreCount.setOrganizationEvaluationId(dto.getOrganizationEvaluationId());
											scoreCount.setOrganizationEvaluationRuleId(dto.getOrganizationEvaluationRuleId());
											scoreCount.setIndicatorId(dto.getIndicatorId());
											for (OrganizationEvaluationInfoResDTO f : indicator){
												if(f.getOptionCode().endsWith("MIN") || f.getOptionCode().endsWith("MAX")){
													f.setQuantifiedValue(map.get(f.getOptionCode()));
												}
												dto.setFormula(dto.getFormula().replace("["+f.getOptionCode()+"]", f.getQuantifiedValue() + ""));
											}
											log.info("计算公式为："+dto.getFormula());
											double score = FormulaUtils.caculateFormula(dto.getFormula());
											if(StringUtils.inStringIgnoreCase("DF",dto.getOptionCode())){
												totalScore = totalScore+score;
											}
											log.info(dto.getOrganizationShortName()+"|"+dto.getIndicatorName()+"|"+dto.getOptionCode()+"----------"+dto.getFormula()+"="+score);
											OrganizationEvaluationInfo info  = baseMapper.selectById(dto.getId());
											//将计算结果保存
											info.setQuantifiedValue(score);
											dto.setQuantifiedValue(score);
											this.saveOrUpdate(info);
										}
									}

									log.info("综合得分："+totalScore);
									if(null==scoreCount.getOrganizationEvaluationId()|| null== scoreCount.getOrganizationEvaluationRuleId() || null == scoreCount.getIndicatorId()){
											continue;
									}
									scoreCount.setScore(totalScore);
									scoreCount.setObversionScore(totalScore);
									scoreCount.setIsQuantified("1");
									List<EvaluationScoreCount>  listcount = iEvaluatioinScoreCountService.getEvaluationScoreCountList(scoreCount.getOrganizationEvaluationId(),scoreCount.getOrganizationEvaluationRuleId(),scoreCount.getIndicatorId());
									if(null  != listcount && listcount.size() >0){
										List<String> ids = listcount.stream().map(EvaluationScoreCount::getId).collect(Collectors.toList());
										boolean cod = iEvaluatioinScoreCountService.removeByIds(ids);
									}
									iEvaluatioinScoreCountService.save(scoreCount);
								}
							}
						}
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
		return true;
	}


	//todo  后续添加需要计算的指标项
	public boolean calculateIdicatorItem(List<OrganizationEvaluationInfo> organizationEvaluationInfos){


		//根据考评规则分组
		Map<String, List<OrganizationEvaluationInfo>> collect1 = organizationEvaluationInfos.stream().collect(Collectors.groupingBy(OrganizationEvaluationInfo::getOrganizationEvaluationRuleId));
		for (Map.Entry<String, List<OrganizationEvaluationInfo>> entry1 : collect1.entrySet()){
			//根据指标id分组
			Map<String, List<OrganizationEvaluationInfo>> collect = entry1.getValue().stream().collect(Collectors.groupingBy(OrganizationEvaluationInfo::getIndicatorId));
			for(Map.Entry<String, List<OrganizationEvaluationInfo>> entry : collect.entrySet()){
				//过滤差值考评指标明细
				List<OrganizationEvaluationInfo> cz = entry.getValue().stream().filter(r -> r.getOptionCode().equals("CZ")).collect(Collectors.toList());
				if (cz.size() == 1){
					List<OrganizationEvaluationInfo> wcz = entry.getValue().stream().filter(r -> r.getOptionCode().equals("WCZ")).collect(Collectors.toList());
					List<OrganizationEvaluationInfo> value = entry.getValue().stream().filter(r -> r.getOptionCode().equals("JHZ") || r.getOptionCode().equals("DBZ") || r.getOptionCode().equals("MBZ")).collect(Collectors.toList());

					//完成值与目标值 ||计划值||对标值进行计算
					if (wcz.size() == 1 && value.size() == 1){
						double wczValue = wcz.get(0).getQuantifiedValue();
						double dbzvalue = value.get(0).getQuantifiedValue();
						cz.get(0).setQuantifiedValue(wczValue - dbzvalue);
						cz.get(0).setCreateTime(new Date());
						boolean b = super.saveOrUpdate(cz.get(0));
						if(!b){
							throw new CustomException("更新失败");
						}
					}
				}

				//过滤单位千瓦利润考评指标明细
				List<OrganizationEvaluationInfo> dwqwlr = entry.getValue().stream().filter(r -> r.getOptionCode().equals("DWQWLR")).collect(Collectors.toList());
				if (dwqwlr.size() == 1){
					List<OrganizationEvaluationInfo> lrgxl = entry.getValue().stream().filter(r -> r.getOptionCode().equals("LRGXL")).collect(Collectors.toList());
					List<OrganizationEvaluationInfo> zjrl = entry.getValue().stream().filter(r -> r.getOptionCode().equals("ZJRL")).collect(Collectors.toList());

					//完成值与目标值 ||计划值||对标值进行计算
					if (lrgxl.size() == 1 && zjrl.size() == 1){
						double wczValue = lrgxl.get(0).getQuantifiedValue();
						double dbzvalue = zjrl.get(0).getQuantifiedValue();
						if (dbzvalue == 0 || dbzvalue == 0.0){
							dwqwlr.get(0).setQuantifiedValue(0);
							dwqwlr.get(0).setCreateTime(new Date());
							boolean b = super.saveOrUpdate(dwqwlr.get(0));
							if(!b){
								throw new CustomException("更新失败");
							}
						}else{
							//double保留两位小数
							double v = new BigDecimal(wczValue / dbzvalue).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
							dwqwlr.get(0).setQuantifiedValue(v);
							dwqwlr.get(0).setCreateTime(new Date());
							boolean b = super.saveOrUpdate(dwqwlr.get(0));
							if(!b){
								throw new CustomException("更新失败");
							}
						}
					}
				}

			}
		}
		return true;
	}
}
